#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include "pipecmd.h"
#include <afblib/strlist.h>

/* generate new pipeline command node */
static pipecmd* new_command(char* cmdname,
      strlist* argv, pipecmd* input) {
   assert(cmdname != 0);
   pipecmd* cmd = (pipecmd*) malloc(sizeof(pipecmd));
   if (cmd == 0) return 0;
   cmd->cmdname = cmdname;
   strlist_push0(argv);
   cmd->argv = argv;
   cmd->input = input;
   return cmd;
}

/* construct pipeline chain out of tokens */
pipecmd* pipe_constructor(strlist* tokens) {
   pipecmd* input = 0;
   char* cmdname = 0;
   strlist* argv = 0;
   for (int index = 0; index < tokens->len; ++index) {
      if (strcmp(tokens->list[index], "|") == 0) {
         if (argv == 0) return 0; /* syntax error */
         input = new_command(cmdname, argv, input);
         cmdname = 0; argv = 0;
      } else {
         if (argv == 0) {
            argv = (strlist*) malloc(sizeof(strlist));
            if (argv == 0) return 0;
            argv->list = 0; argv->len = 0; argv->allocated = 0;
         }
         if (cmdname == 0) cmdname = tokens->list[index];
         if (!strlist_push(argv, tokens->list[index])) return 0;
      }
   }
   if (cmdname != 0) input = new_command(cmdname, argv, input);
   return input;
}

/* free space allocated by pipe_constructor() */
void pipe_free(pipecmd* cmd) {
   if (cmd->input) pipe_free(cmd->input);
   strlist_free(cmd->argv); free(cmd->argv);
   free(cmd);
}
